@@ -27,4 +27,4 @@
 ;;; Code:
 
 (define %version
-  "0.1")
+  "0.2.0")
