@@ -180,7 +180,8 @@ character."
 (define read-backquoted-command
   ;; A procedure for reading a backquoted command (e.g, "`command`").
   ;; This is parameterized to avoid a circular dependency.
-  (make-parameter (lambda (port) (throw 'backquoted-command-parser-unset))))
+  (make-parameter (lambda* (port #:key quoted?)
+                    (throw 'backquoted-command-parser-unset))))
 
 (define* (get-parameter port #:key (multidigit? #f))
   "Get a parameter name (excluding the leading '$') from @var{port}.
@@ -328,24 +329,27 @@ leading '$')."
        (match (get-char port)
          (#\) `(<sh-cmd-sub> ,@result)))))))
 
-(define (get-backquoted-command port)
-  "Get a backquoted command ('`...`') from @var{port}."
+(define* (get-backquoted-command port #:key quoted?)
+  "Get a backquoted command ('`...`') from @var{port}.  If
+@var{quoted?} is set, treat the backquoted command as if it were
+quoted."
   (match (get-char port)
     (#\`
-     (let ((result ((read-backquoted-command) port)))
+     (let ((result ((read-backquoted-command) port #:quoted? quoted?)))
        (match (get-char port)
          (#\` `(<sh-cmd-sub> ,@result)))))))
 
-(define (get-expansion port)
+(define* (get-expansion port #:key quoted?)
   "Get an expansion ('$name', '${...}', '$(...)', or '`...`') from
-@var{port}."
+@var{port}.  If @var{quoted?} is set, treat backquoted commands as if
+they were quoted."
   (match (lookahead-char port)
     (#\$ (begin
            (get-char port)
            (match (lookahead-char port)
              (#\( (get-bracketed-command port))
              (_ (get-parameter-expansion port)))))
-    (#\` (get-backquoted-command port))))
+    (#\` (get-backquoted-command port #:quoted? quoted?))))
 
 ;; When this parameter is true, expansion processing is enabled.
 (define expansions? (make-parameter #t))
@@ -395,7 +399,7 @@ next character statisfies @var{pred} (or is a newline)."
                                 (words words)))))
          ((or #\$ #\`)
           (if (expansions?)
-              (let ((expansion (get-expansion port)))
+              (let ((expansion (get-expansion port #:quoted? #t)))
                 (loop (lookahead-char port)
                       (cons (or expansion (string chr)) acc)))
               (loop (next-char port) (cons (string chr) acc))))
