@@ -398,6 +398,8 @@ the same number of times.)"
     (LPAREN! pattern RPAREN! compound-list DSEMI linebreak)
     : `(,$2 ,@$4))
 
+   ;; If this rule is updated, the hooked version given below must be
+   ;; updated as well.
    (pattern
     (WORD*-without-Esac)
     : `(,$1)
@@ -683,11 +685,14 @@ the same number of times.)"
 
    ;; Sometimes a "pattern" non-terminal comes before an unbalanced
    ;; "RPAREN". This reduction hook can be used to pretend that we
-   ;; encountered an "LPAREN".
+   ;; encountered an "LPAREN".  It should match the unhooked one given
+   ;; above.
 
    (pattern!
-    (pattern)
-    : (begin (open-bracket-hook) $1))))
+    (WORD*-without-Esac)
+    : (begin (open-bracket-hook) `(,$1))
+    (pattern! PIPE WORD*)
+    : (append $1 `(,$3)))))
 
 (define* (syntax-error message #:optional token)
   "Handle a parser error"
@@ -750,10 +755,17 @@ ignored."
     (((? symbol? tag) . rest) `((,tag . ,rest)))
     (code code)))
 
-(define (call-with-backquoted-input-port port proc)
+(define* (call-with-backquoted-input-port port proc #:key quoted?)
   "Call @var{proc} with a wrapped version of @var{port} that will
 return the end-of-file object upon encountering an unescaped backquote
-\"`\" (without consuming the backquote)."
+\"`\" (without consuming the backquote).  If @var{quoted?} is set,
+treat the double quote character as escapable."
+  (define (escape-char? chr)
+    (or (char=? chr #\$)
+        (char=? chr #\`)
+        (char=? chr #\\)
+        (and quoted? (char=? chr #\"))))
+
   (define wrapped-port
     (make-soft-port
      (vector
@@ -766,7 +778,7 @@ return the end-of-file object upon encountering an unescaped backquote
           (#\\ (begin
                  (get-char port)
                  (match (lookahead-char port)
-                   ((or #\$ #\` #\\) (get-char port))
+                   ((? escape-char?) (get-char port))
                    (_ #\\))))
           (_ (get-char port))))
       ;; close-port
@@ -796,11 +808,14 @@ bracket."
                          #:open-bracket-hook incr-bracket-depth!
                          #:close-bracket-hook decr-bracket-depth!)))
 
-(define (read-sh/backquoted port)
-  "Read Shell code from @var{port} until the first unescaped backquote."
+(define* (read-sh/backquoted port #:key quoted?)
+  "Read Shell code from @var{port} until the first unescaped
+backquote.  If @var{quoted?} is set, treat the double quote character
+as escapable."
   (call-with-backquoted-input-port port
     (lambda (port)
-      (->command-list (parse port)))))
+      (->command-list (parse port)))
+    #:quoted? quoted?))
 
 (define* (read-sh #:optional (port #f))
   "Read a complete Shell command from @var{port} (or the current input
